###################################################
### chunk number 1: preliminaries
###################################################
library(ReacTran)
options(prompt = " ")


###################################################
### chunk number 2: 
###################################################
Grid <- setup.grid.1D(N = 1000, L = 10)


###################################################
### chunk number 3: 
###################################################
r  <- setup.prop.1D(grid = Grid, func = function(r) r)
r2 <- setup.prop.1D(grid = Grid, func = function(r) r^2)


###################################################
### chunk number 4: 
###################################################
library(ReacTran)
pde1D <-function(t, C, parms, A = 1)  {
  tran   <- tran.1D (C = C, A = A, D = D, C.down = Cext, dx = Grid)$dC
  list(tran - Q)  # the return value: rate of change
}


###################################################
### chunk number 5: 
###################################################
D    <- 1    # diffusion constant
Q    <- 1    # uptake rate
Cext <- 20


###################################################
### chunk number 6: 
###################################################
library(rootSolve)
Cartesian   <- steady.1D(y = runif(Grid$N),
  func = pde1D, parms = NULL, nspec = 1, A = 1)
Cylindrical <- steady.1D(y = runif(Grid$N),
  func = pde1D, parms = NULL, nspec = 1, A = r)


###################################################
### chunk number 7: 
###################################################
print(system.time(
  Spherical   <- steady.1D(y = runif(Grid$N),
    func = pde1D, parms = NULL, nspec = 1, A = r2)
))


###################################################
### chunk number 8: pde
###################################################
plot (Grid$x.mid, Cartesian$y, type = "l", main = "steady-state PDE",
  lwd = 3, xlab = "x", ylab = "C", col = "darkgreen", lty = 1)
lines(Grid$x.mid, Cylindrical$y, lwd = 3, col="blue", lty = 2)
lines(Grid$x.mid, Spherical$y, lwd = 3, col = "red", lty = 3)
legend("bottomright", c("cartesian", "cylindrical", "spherical"),
  col = c("darkgreen", "blue", "red"), lwd = 3, lty = 1:3)


###################################################
### chunk number 9: figpde
###################################################
plot (Grid$x.mid, Cartesian$y, type = "l", main = "steady-state PDE",
  lwd = 3, xlab = "x", ylab = "C", col = "darkgreen", lty = 1)
lines(Grid$x.mid, Cylindrical$y, lwd = 3, col="blue", lty = 2)
lines(Grid$x.mid, Spherical$y, lwd = 3, col = "red", lty = 3)
legend("bottomright", c("cartesian", "cylindrical", "spherical"),
  col = c("darkgreen", "blue", "red"), lwd = 3, lty = 1:3)


###################################################
### chunk number 10: 
###################################################
max(abs(Q/6/D*(r2$mid - 10^2) + Cext - Spherical$y))
max(abs(Q/4/D*(r2$mid - 10^2) + Cext - Cylindrical$y))
max(abs(Q/2/D*(r2$mid - 10^2) + Cext - Cartesian$y))


###################################################
### chunk number 11: 
###################################################
require(deSolve)
times <- seq(0, 100, by = 1)
system.time(
  out <- ode.1D(y = rep(1, Grid$N), times = times, func = pde1D,
    parms = NULL, nspec = 1, A = r2)
)


###################################################
### chunk number 12: 
###################################################
tail(out[, 1:4], n = 3)


###################################################
### chunk number 13: yy
###################################################
image(out, grid = Grid$x.mid, xlab = "time, days", 
     ylab = "Distance, cm", main = "PDE", add.contour = TRUE)


###################################################
### chunk number 14: 
###################################################
dx    <- 0.2
xgrid <- setup.grid.1D(-100, 100, dx.1 = dx)
x     <- xgrid$x.mid
N     <- xgrid$N


###################################################
### chunk number 15: 
###################################################
uini  <- exp(-0.05 * x^2)
vini  <- rep(0, N)
yini  <- c(uini, vini)
times <- seq (from = 0, to = 50, by = 1)


###################################################
### chunk number 16: 
###################################################
wave <- function (t, y, parms) {
  u1 <- y[1:N]
  u2 <- y[-(1:N)]

  du1 <- u2
  du2 <- tran.1D(C = u1, C.up = 0, C.down = 0, D = 1, dx = xgrid)$dC
  return(list(c(du1, du2)))
}


###################################################
### chunk number 17: 
###################################################
out <- ode.1D(func = wave, y = yini, times = times, parms = NULL,
         nspec = 2, method = "ode45", dimens = N, names = c("u", "v"))


###################################################
### chunk number 18: wave
###################################################
u <- out[  ,2:(N+1)]

plot(x, u[1,], xlab = "x", ylab = "u", type = "l", lwd = 2, 
     ylim = range(u), xlim = range(-50, 50))

iout <- seq(11, length(times), by=10)
for(i in iout)
  lines(x, u[i,], lwd = 2, col = "darkgrey", lty = i/10)

legend("topright", lty = 1:5, lwd = 2, col = "darkgrey", 
       legend=paste("t = ", times[iout]))


###################################################
### chunk number 19: wave
###################################################
u <- out[  ,2:(N+1)]

plot(x, u[1,], xlab = "x", ylab = "u", type = "l", lwd = 2, 
     ylim = range(u), xlim = range(-50, 50))

iout <- seq(11, length(times), by=10)
for(i in iout)
  lines(x, u[i,], lwd = 2, col = "darkgrey", lty = i/10)

legend("topright", lty = 1:5, lwd = 2, col = "darkgrey", 
       legend=paste("t = ", times[iout]))


###################################################
### chunk number 20: waveimg
###################################################
par(mar=c(0,0,0,0))
image(out, which = "u", method = "persp", main = "",
        border = NA, col = "lightblue", box = FALSE, shade = 0.5, theta = 0, 
        phi = 60)


###################################################
### chunk number 21: 
###################################################
require(ReacTran)
pde2D <- function (t, y, parms) {
  CONC <- matrix(nr = n, nc = n, y)
  Tran <- tran.2D(CONC, D.x = Dx, D.y = Dy, dx = dx,  dy = dy)
  dCONC <- Tran$dC - r * CONC
  dCONC[ii]<- dCONC[ii] + p
  return(list(as.vector(dCONC)))
}


###################################################
### chunk number 22: 
###################################################
n  <- 100
dy <- dx <- 1
Dy <- Dx <- 2
r  <- 0.001
p  <- runif(50)
ii <- trunc(cbind(runif(50)*n, runif(50)*n) + 1)


###################################################
### chunk number 23: 
###################################################
require(rootSolve)
Conc0 <- matrix(nr = n, nc = n, 10.)
print(system.time(
  ST <- steady.2D(y = Conc0, func = pde2D, parms = NULL, dimens = c(n, n),
   lrw = 600000)
))


###################################################
### chunk number 24: D2
###################################################
image(ST, main = "steady-state 2-D PDE")


###################################################
### chunk number 25: D2fig
###################################################
image(ST, main = "steady-state 2-D PDE")


